﻿#region Copyright (c) 2008 S. van Deursen
/* The CuttingEdge.Logging library allows developers to plug a logging mechanism into their web- and desktop
 * applications.
 * 
 * Copyright (C) 2008 S. van Deursen
 * 
 * To contact me, please visit my blog at http://www.cuttingedge.it/blogs/steven/ or mail to steven at 
 * cuttingedge.it.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
 * associated documentation files (the "Software"), to deal in the Software without restriction, including 
 * without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the 
 * following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all copies or substantial 
 * portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
 * LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO 
 * EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER 
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE 
 * USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#endregion

using System;
using System.ComponentModel;

namespace CuttingEdge.Logging
{
    /// <summary>
    /// Defines a method that logs a given message with type and optional source and exception to the logging
    /// store.
    /// </summary>
    /// <remarks>
    /// The <see cref="LoggerExtensions"/> class defines convenient extension methods for the 
    /// <see cref="ILogger"/> interface.
    /// </remarks>
    public interface ILogger
    {
        /// <summary>Logs the specified entry.</summary>
        /// <param name="entry">The entry to log.</param>
        /// <remarks>
        /// Implementations of this method must guarantee to be thread safe.
        /// </remarks>
        /// <returns>
        /// The id of the saved log (or null when an id is not appropriate for this type of logger.
        /// </returns>
        /// <exception cref="ArgumentNullException">
        /// Thrown when the given <paramref name="entry"/> is a null reference.
        /// </exception>
        object Log(LogEntry entry);
    }
}
